# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from . import runtime_env_agent_pb2 as src_dot_ray_dot_protobuf_dot_runtime__env__agent__pb2


class RuntimeEnvServiceStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.CreateRuntimeEnv = channel.unary_unary(
        '/ray.rpc.RuntimeEnvService/CreateRuntimeEnv',
        request_serializer=src_dot_ray_dot_protobuf_dot_runtime__env__agent__pb2.CreateRuntimeEnvRequest.SerializeToString,
        response_deserializer=src_dot_ray_dot_protobuf_dot_runtime__env__agent__pb2.CreateRuntimeEnvReply.FromString,
        )
    self.DeleteRuntimeEnv = channel.unary_unary(
        '/ray.rpc.RuntimeEnvService/DeleteRuntimeEnv',
        request_serializer=src_dot_ray_dot_protobuf_dot_runtime__env__agent__pb2.DeleteRuntimeEnvRequest.SerializeToString,
        response_deserializer=src_dot_ray_dot_protobuf_dot_runtime__env__agent__pb2.DeleteRuntimeEnvReply.FromString,
        )


class RuntimeEnvServiceServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def CreateRuntimeEnv(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DeleteRuntimeEnv(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_RuntimeEnvServiceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'CreateRuntimeEnv': grpc.unary_unary_rpc_method_handler(
          servicer.CreateRuntimeEnv,
          request_deserializer=src_dot_ray_dot_protobuf_dot_runtime__env__agent__pb2.CreateRuntimeEnvRequest.FromString,
          response_serializer=src_dot_ray_dot_protobuf_dot_runtime__env__agent__pb2.CreateRuntimeEnvReply.SerializeToString,
      ),
      'DeleteRuntimeEnv': grpc.unary_unary_rpc_method_handler(
          servicer.DeleteRuntimeEnv,
          request_deserializer=src_dot_ray_dot_protobuf_dot_runtime__env__agent__pb2.DeleteRuntimeEnvRequest.FromString,
          response_serializer=src_dot_ray_dot_protobuf_dot_runtime__env__agent__pb2.DeleteRuntimeEnvReply.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'ray.rpc.RuntimeEnvService', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))
